@@ -7,8 +7,7 @@ import torch.nn.functional as F
 from scipy.ndimage.morphology import distance_transform_edt
 from skimage import measure
 
-from neuralnets.data.datasets import StronglyLabeledStandardDataset, StronglyLabeledVolumeDataset, \
-    StronglyLabeledMultiVolumeDataset
+from neuralnets.data.datasets import LabeledStandardDataset, LabeledVolumeDataset, LabeledMultiVolumeDataset
 
 from neuralnets.util.tools import tensor_to_device
 
@@ -33,19 +32,13 @@ class CrossEntropyLoss(nn.Module):
         w = None
         if weight is not None:
             w = tensor_to_device(torch.Tensor(weight), device=device)
-        self.ce = nn.CrossEntropyLoss(weight=w, reduction="none")
+        self.ce = nn.CrossEntropyLoss(weight=w, ignore_index=255)
 
-    def forward(self, logits, target, mask=None):
+    def forward(self, logits, target):
 
         # compute loss unreduced and reshape to vector
         loss = self.ce(logits, target).view(-1)
 
-        # size averaging if necessary
-        if mask is not None:
-            loss = loss[mask.view(-1)].mean()
-        else:
-            loss = loss.mean()
-
         return loss
 
     def set_weight(self, weight=None):
@@ -420,19 +413,19 @@ def get_balancing_weights(dataset):
     Returns a set of balancing weights for a specific labeled dataset
 
     :param dataset: labeled dataset instance of
-                        - neuralnets.data.datasets.StronglyLabeledStandardDataset
-                        - neuralnets.data.datasets.StronglyLabeledVolumeDataset
-                        - neuralnets.data.datasets.StronglyLabeledMultiVolumeDataset
+                        - neuralnets.data.datasets.LabeledStandardDataset
+                        - neuralnets.data.datasets.LabeledVolumeDataset
+                        - neuralnets.data.datasets.LabeledMultiVolumeDataset
     :return: a tuple of balancing weights, if an unsuitable object is provided, it returns None
     """
 
-    if isinstance(dataset, StronglyLabeledStandardDataset) or isinstance(dataset, StronglyLabeledVolumeDataset):
+    if isinstance(dataset, LabeledStandardDataset) or isinstance(dataset, LabeledVolumeDataset):
         weight = np.zeros((len(dataset.coi)))
         for i, c in enumerate(dataset.coi):
             weight[i] = 1 / np.count_nonzero(dataset.labels == c)
         weight = weight / np.sum(weight)
         return tuple(weight)
-    elif isinstance(dataset, StronglyLabeledMultiVolumeDataset):
+    elif isinstance(dataset, LabeledMultiVolumeDataset):
         freq = np.zeros((len(dataset.coi)))
         for i, c in enumerate(dataset.coi):
             for labels in dataset.labels:
