@@ -1,4 +1,3 @@
-import copy
 import random
 
 import numpy as np
@@ -42,7 +41,7 @@ def sample_labeled_input(data, labels, input_shape, preloaded=True, type='pngseq
         input = input[:, x:x + input_shape[1], y:y + input_shape[2]]
         target = target[:, x:x + input_shape[1], y:y + input_shape[2]]
 
-    return copy.copy(input), copy.copy(target)
+    return input.copy(), target.copy()
 
 
 def sample_unlabeled_input(data, input_shape, preloaded=True, type='pngseq', data_shape=None):
@@ -74,7 +73,7 @@ def sample_unlabeled_input(data, input_shape, preloaded=True, type='pngseq', dat
         input = read_volume(data, type=type, start=z, stop=z + input_shape[0])
         input = input[:, x:x + input_shape[1], y:y + input_shape[2]]
 
-    return copy.copy(input)
+    return input.copy()
 
 
 def gaussian_window(size, sigma=1):
@@ -114,6 +113,36 @@ def gaussian_window(size, sigma=1):
     return gw
 
 
+def parse_params(params):
+    """
+    Parse a YAML parameter dictionary
+
+    :param params: dictionary containing the parameters
+    :return: parsed dictionary
+    """
+
+    keys = params.keys()
+
+    if 'input_size' in keys:
+        params['input_size'] = [int(item) for item in params['input_size'].split(',')]
+    if 'coi' in keys:
+        params['coi'] = [int(c) for c in params['coi'].split(',')]
+    if 'orientations' in keys:
+        if type(params['orientations']) is int:
+            params['orientations'] = [params['orientations']]
+        else:
+            params['orientations'] = [int(c) for c in params['orientations'].split(',')]
+    if 'gpus' in keys:
+        if type(params['gpus']) is int:
+            params['gpus'] = [params['gpus']]
+        else:
+            params['gpus'] = [int(c) for c in params['gpus'].split(',')]
+    if 'log_refresh_rate' in keys:
+        params['log_refresh_rate'] = None if params['log_refresh_rate'] < 0 else int(params['log_refresh_rate'])
+
+    return params
+
+
 def load_unet(state_dict, device=0):
     """
     Load a pretrained pytorch U-Net state dict
